@@ -5,6 +5,8 @@ require 'open3'
 require 'json'
 
 require 'vagrant-libvirt/util/byte_number'
+require 'vagrant-libvirt/util/storage_util'
+require 'vagrant-libvirt/util/ui'
 
 module VagrantPlugins
   module ProviderLibvirt
@@ -134,7 +136,7 @@ module VagrantPlugins
                       ''
                     end
 
-          if version.empty?
+          if version.empty? || version == '0'
             ui.warn(I18n.t('vagrant_libvirt.box_version_missing', name: box.name.to_s))
 
             version = "0_#{File.mtime(path).to_i}"
