@@ -12,8 +12,7 @@ module VagrantPlugins
           env[:ui].info(I18n.t(
             "vagrant.actions.vm.snapshot.saving",
             name: env[:snapshot_name]))
-          env[:machine].provider.driver.create_snapshot(
-            env[:machine], env[:snapshot_name])
+          env[:machine].provider.driver.create_snapshot(env[:snapshot_name])
 
           env[:ui].success(I18n.t(
             "vagrant.actions.vm.snapshot.saved",
