@@ -109,7 +109,7 @@ module VagrantPlugins
       end
 
       class FogCreateServerError < VagrantLibvirtError
-        error_key(:fog_create_server_error)
+        error_key(:create_server_error)
       end
 
       # Network exceptions
@@ -154,6 +154,10 @@ module VagrantPlugins
       end
 
       # Other exceptions
+      class UpdateServerError < VagrantLibvirtError
+        error_key(:create_server_error)
+      end
+
       class InterfaceSlotNotAvailable < VagrantLibvirtError
         error_key(:interface_slot_not_available)
       end
@@ -174,6 +178,10 @@ module VagrantPlugins
         error_key(:no_domain_error)
       end
 
+      class DomainStartError < VagrantLibvirtError
+        error_key(:domain_start_error)
+      end
+
       class AttachDeviceError < VagrantLibvirtError
         error_key(:attach_device_error)
       end
@@ -186,8 +194,24 @@ module VagrantPlugins
         error_key(:no_ip_address_error)
       end
 
-      class DeleteSnapshotError < VagrantLibvirtError
-        error_key(:delete_snapshot_error)
+      class SnapshotMissing < VagrantLibvirtError
+        error_key(:snapshot_missing)
+      end
+
+      class SnapshotDeletionError < VagrantLibvirtError
+        error_key(:snapshot_deletion_error)
+      end
+
+      class SnapshotListError < VagrantLibvirtError
+        error_key(:snapshot_list_error)
+      end
+
+      class SnapshotCreationError < VagrantLibvirtError
+        error_key(:snapshot_creation_error)
+      end
+
+      class SnapshotReversionError < VagrantLibvirtError
+        error_key(:snapshot_reversion_error)
       end
 
       class SerialCannotCreatePathError < VagrantLibvirtError
